package com.seaway.flutter.plugin.device;

import android.os.Build;

import com.google.gson.Gson;
import com.seaway.flutter.plugin.device.util.Util;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * SwDeviceInfoPlugin
 */
public class SwDeviceInfoPlugin implements MethodCallHandler {
    private Registrar registrar;
    private DeviceInfo deviceInfo;

    public SwDeviceInfoPlugin(Registrar registrar) {
        this.registrar = registrar;
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "sw_device_info");
        channel.setMethodCallHandler(new SwDeviceInfoPlugin(registrar));
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        if (call.method.equals("getDeviceInfo")) {
            if (null == deviceInfo) {
                deviceInfo = new DeviceInfo();
                deviceInfo.setDeviceMfrs(Build.MANUFACTURER);
                deviceInfo.setDeviceModel(Build.MODEL);
                deviceInfo.setProvider(Util.getProvideName(registrar.context()));

                int[] screenInfo = Util.getScreenInfo(registrar.context());
                deviceInfo.setWidth(String.valueOf(screenInfo[0]));
                deviceInfo.setHeight(String.valueOf(screenInfo[1]));
                deviceInfo.setOsVersion(Build.VERSION.RELEASE);
                deviceInfo.setDeviceNo(Util.getDeviceId(registrar.context()));
                deviceInfo.setSignMd5(Util.getApplicationSignMd5(registrar.context()));
                deviceInfo.setVersionName(Util.getVersioName(registrar.context()));
                deviceInfo.setOsName("A");
            }

            result.success(new Gson().toJson(deviceInfo));
        } else {
            result.notImplemented();
        }
    }
}
