#import "SwDeviceInfoPlugin.h"
#import "KeychainItemWrapper.h"

#import <CoreTelephony/CTTelephonyNetworkInfo.h>
#import <CoreTelephony/CTCarrier.h>

#define kDeviceInfoUDIDKey [[NSBundle mainBundle] bundleIdentifier]
#define kDeviceInfo @"{\"deviceMfrs\":\"Apple\",\"deviceModel\":\"%@\",\"provider\":\"%@\",\"width\":\"%.f\",\"height\":\"%.f\",\"osVersion\":\"%@\",\"deviceNo\":\"%@\",\"versionName\":\"%@\",\"osName\":\"I\"}"

@implementation SwDeviceInfoPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"sw_device_info"
            binaryMessenger:[registrar messenger]];
  SwDeviceInfoPlugin* instance = [[SwDeviceInfoPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getDeviceInfo" isEqualToString:call.method]) {
    //(@"{\"deviceMfrs":\"Apple\"}");
//      @{@""}
      result([NSString stringWithFormat:kDeviceInfo,
              [UIDevice currentDevice].model,//deviceModel
              self.provider,//provider
              [UIScreen mainScreen].bounds.size.width * [UIScreen mainScreen].scale,//width
              [UIScreen mainScreen].bounds.size.height * [UIScreen mainScreen].scale,//height
              [[UIDevice currentDevice] systemVersion],//osVersion
             self.deviceNo,//deviceNo
              self.version]
             );
//      result(@{@"deviceMfrs":});
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (NSString *)provider {
    
    if (!_provider) {
        
        NSString * providerName= [[[[CTTelephonyNetworkInfo alloc] init] subscriberCellularProvider] carrierName];
        
        if ([providerName isEqualToString:@"中国移动"]) {
            
            _provider = @"0";
        }else if ([providerName isEqualToString:@"中国联通"]){
            
            _provider = @"1";
        }else if ([providerName isEqualToString:@"中国电信"]){
            
            _provider = @"2";
        }else {
            _provider = providerName;
        }
    }
    return _provider;
}

- (NSString *)deviceNo {
    
    if (!_deviceNo) {
        
        KeychainItemWrapper *wrapper = [[KeychainItemWrapper alloc] initWithIdentifier:kDeviceInfoUDIDKey accessGroup:nil];
        
        _deviceNo = [wrapper objectForKey:(__bridge_transfer id)kSecAttrAccount];
        
        if (_deviceNo.length == 0) {
            //生成UUID 并保存至Keychain(支持iOS7+)
            CFUUIDRef uuid = CFUUIDCreate(nil);
            CFStringRef uuidStr = CFUUIDCreateString(nil, uuid);
            _deviceNo = CFBridgingRelease(CFStringCreateCopy(NULL, uuidStr));
            CFRelease(uuid);
            CFRelease(uuidStr);
            
            [wrapper setObject:_deviceNo forKey:(__bridge_transfer id)kSecAttrAccount];
        }
    }
    return _deviceNo;
}

- (NSString *)version {
    if (!_version) {
        NSString *version = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
        NSString *build = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
        _version = [NSString stringWithFormat:@"%@.%@",version,build];
    }
    
    return _version;
}

@end
