package com.seaway.flutter.plugin.device.util;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.support.v4.content.ContextCompat;
import android.telephony.TelephonyManager;
import android.util.DisplayMetrics;
import android.util.Log;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public class Util {
    /**
     * 获取运营商
     *
     * @return 运营商<br>0：中国移动；1：中国联通；2：中国电信；3：未知
     */
    public static String getProvideName(Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && (ContextCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED)) {
            return "3";
        }
        try {
            TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
            String operator = tm.getSimOperator();
            if (null != operator) {
                if (operator.equals("46000") || operator.equals("46002")) {
                    // 中国移动
                    return "0";
                } else if (operator.equals("46001")) {
                    // 中国联通
                    return "1";
                } else if (operator.equals("46003")) {
                    // 中国电信
                    return "2";
                }
            }
        } catch (Exception e) {
        }
        return "3";
    }

    /**
     * 获取屏幕信息
     *
     * @return 屏幕宽高，[0]：宽度；[1]：高度
     */
    public static int[] getScreenInfo(Context context) {
        DisplayMetrics metric = context.getResources().getDisplayMetrics();
        int width = metric.widthPixels;
        int height = metric.heightPixels;
        return new int[]{width, height};
    }

    /**
     * 获取设备编号
     *
     * @return 设备编号
     */
    public static String getDeviceId(Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && (ContextCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED)) {
            return "";
        }

        TelephonyManager tm = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
        return tm.getDeviceId();
    }

    /**
     * 获取应用签名的MD5值
     *
     * @return
     */
    public static String getApplicationSignMd5(Context context) {
        String signValidString = null;
        try {
            PackageInfo packageInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), PackageManager.GET_SIGNATURES);
            signValidString = getSignValidString(packageInfo.signatures[0].toByteArray());
            Log.i("TAG","signValidString is : " + signValidString);
        } catch (Exception e) {
            Log.e("TAG","",e);
            e.printStackTrace();
        }

        return signValidString;
    }

    private static String getSignValidString(byte[] paramArrayOfByte) throws NoSuchAlgorithmException {
        MessageDigest localMessageDigest = MessageDigest.getInstance("MD5");
        localMessageDigest.update(paramArrayOfByte);
        return toHexString(localMessageDigest.digest());
    }

    private static String toHexString(byte[] paramArrayOfByte) {
        if (paramArrayOfByte == null) {
            return null;
        }
        StringBuilder localStringBuilder = new StringBuilder(2 * paramArrayOfByte.length);
        for (int i = 0; ; i++) {
            if (i >= paramArrayOfByte.length) {
                return localStringBuilder.toString();
            }
            String str = Integer.toString(0xFF & paramArrayOfByte[i], 16);
            if (str.length() == 1) {
                str = "0" + str;
            }
            localStringBuilder.append(str);
        }
    }

    /**
     * 获取客户端版本号
     * @param context
     * @return
     */
    public static String getVersioName(Context context) {
        try {
            PackageManager manager = context.getPackageManager();
            PackageInfo info = manager.getPackageInfo(context.getPackageName(),0);
            return info.versionName;
        } catch (Exception e) {
            return "";
        }
    }
}
